@@ -61,11 +61,13 @@ QHash<int,QByteArray> UpdateModel::roleNames() const
     return QAbstractItemModel::roleNames().unite({
         { Qt::CheckStateRole, "checked" },
         { ResourceProgressRole, "resourceProgress" },
+        { ResourceStateRole, "resourceState" },
         { ResourceRole, "resource" },
         { SizeRole, "size" },
-        { VersionRole, "version" },
         { SectionRole, "section" },
-        { ChangelogRole, "changelog" }
+        { ChangelogRole, "changelog" },
+        { InstalledVersionRole, "installedVersion" },
+        { AvailableVersionRole, "availableVersion" }
     } );
 }
 
@@ -83,15 +85,16 @@ void UpdateModel::setBackend(ResourcesUpdatesModel* updates)
     activityChanged();
 }
 
-void UpdateModel::resourceHasProgressed(AbstractResource* res, qreal progress)
+void UpdateModel::resourceHasProgressed(AbstractResource* res, qreal progress, AbstractBackendUpdater::State state)
 {
     UpdateItem* item = itemFromResource(res);
     if (!item)
         return;
     item->setProgress(progress);
+    item->setState(state);
 
     const QModelIndex idx = indexFromItem(item);
-    Q_EMIT dataChanged(idx, idx, { ResourceProgressRole, SectionResourceProgressRole });
+    Q_EMIT dataChanged(idx, idx, { ResourceProgressRole, ResourceStateRole, SectionResourceProgressRole });
 }
 
 void UpdateModel::activityChanged()
@@ -124,27 +127,33 @@ QVariant UpdateModel::data(const QModelIndex &index, int role) const
         return item->icon();
     case Qt::CheckStateRole:
         return item->checked();
-    case VersionRole:
-        return item->version();
+    case InstalledVersionRole:
+        return item->installedVersion();
+    case AvailableVersionRole:
+        return item->availableVersion();
     case SizeRole:
         return KFormat().formatByteSize(item->size());
     case ResourceRole:
         return QVariant::fromValue<QObject*>(item->resource());
     case ResourceProgressRole:
         return item->progress();
+    case ResourceStateRole:
+        return item->state();
     case ChangelogRole:
         return item->changelog();
     case SectionRole: {
         static const QString appUpdatesSection = i18nc("@item:inlistbox", "Application Updates");
         static const QString systemUpdateSection = i18nc("@item:inlistbox", "System Updates");
-        switch(item->section()) {
-            case UpdateItem::ApplicationSection: return appUpdatesSection;
-            case UpdateItem::SystemSection: return systemUpdateSection;
+        static const QString addonsSection = i18nc("@item:inlistbox", "Addons");
+        switch(item->resource()->type()) {
+            case AbstractResource::Application: return appUpdatesSection;
+            case AbstractResource::Technical: return systemUpdateSection;
+            case AbstractResource::Addon: return addonsSection;
         }
-        return {};
-    } 
+        Q_UNREACHABLE();
+    }
     case SectionResourceProgressRole:
-        return (100-item->progress()) + (101 * item->section());
+        return (100-item->progress()) + (101 * item->resource()->type());
     default:
         break;
     }
@@ -229,24 +238,29 @@ void UpdateModel::setResources(const QList<AbstractResource*>& resources)
     qDeleteAll(m_updateItems);
     m_updateItems.clear();
 
-    QVector<UpdateItem*> appItems, systemItems;
+    QVector<UpdateItem*> appItems, systemItems, addonItems;
     foreach(AbstractResource* res, resources) {
         connect(res, &AbstractResource::changelogFetched, this, &UpdateModel::integrateChangelog, Qt::UniqueConnection);
 
         UpdateItem *updateItem = new UpdateItem(res);
 
-        if(!res->isTechnical()) {
-            updateItem->setSection(UpdateItem::ApplicationSection);
-            appItems += updateItem;
-        } else {
-            updateItem->setSection(UpdateItem::SystemSection);
-            systemItems += updateItem;
+        switch(res->type()) {
+            case AbstractResource::Technical:
+                systemItems += updateItem;
+                break;
+            case AbstractResource::Application:
+                appItems += updateItem;
+                break;
+            case AbstractResource::Addon:
+                addonItems += updateItem;
+                break;
         }
     }
     const auto sortUpdateItems = [](UpdateItem *a, UpdateItem *b) { return a->name() < b->name(); };
-    qSort(appItems.begin(), appItems.end(), sortUpdateItems);
-    qSort(systemItems.begin(), systemItems.end(), sortUpdateItems);
-    m_updateItems = (QVector<UpdateItem*>() << appItems << systemItems);
+    std::sort(appItems.begin(), appItems.end(), sortUpdateItems);
+    std::sort(systemItems.begin(), systemItems.end(), sortUpdateItems);
+    std::sort(addonItems.begin(), addonItems.end(), sortUpdateItems);
+    m_updateItems = (QVector<UpdateItem*>() << appItems << addonItems << systemItems);
     endResetModel();
 
     Q_EMIT hasUpdatesChanged(!resources.isEmpty());
@@ -325,9 +339,23 @@ void UpdateModel::resourceDataChanged(AbstractResource* res, const QVector<QByte
 
     const auto index = indexFromItem(item);
     if (properties.contains("state"))
-        Q_EMIT dataChanged(index, index, {SizeRole, VersionRole});
+        Q_EMIT dataChanged(index, index, {SizeRole, AvailableVersionRole});
     else if (properties.contains("size")) {
         Q_EMIT dataChanged(index, index, {SizeRole});
         m_updateSizeTimer->start();
     }
 }
+
+void UpdateModel::checkAll()
+{
+    for(int i=0, c=rowCount(); i<c; ++i)
+        if(index(i,0).data(Qt::CheckStateRole) != Qt::Checked)
+            setData(index(i,0), Qt::Checked, Qt::CheckStateRole);
+}
+
+void UpdateModel::uncheckAll()
+{
+    for(int i=0, c=rowCount(); i<c; ++i)
+        if(index(i,0).data(Qt::CheckStateRole) != Qt::Unchecked)
+            setData(index(i,0), Qt::Unchecked, Qt::CheckStateRole);
+}
