@@ -20,6 +20,7 @@
  ***************************************************************************/
 
 #include "OdrsReviewsBackend.h"
+#include "AppStreamIntegration.h"
 
 #include <ReviewsBackend/Review.h>
 #include <ReviewsBackend/Rating.h>
@@ -29,10 +30,11 @@
 
 #include <KIO/FileCopyJob>
 #include <KUser>
+#include <KLocalizedString>
 
 #include <QCryptographicHash>
 #include <QDir>
-#include <QDebug>
+#include "libdiscover_debug.h"
 #include <QJsonArray>
 #include <QJsonDocument>
 #include <QJsonObject>
@@ -46,10 +48,9 @@
 // #define APIURL "http://127.0.0.1:5000/1.0/reviews/api"
 #define APIURL "https://odrs.gnome.org/1.0/reviews/api"
 
-OdrsReviewsBackend::OdrsReviewsBackend(AbstractResourcesBackend *parent)
-    : AbstractReviewsBackend(parent)
+OdrsReviewsBackend::OdrsReviewsBackend()
+    : AbstractReviewsBackend(nullptr)
     , m_isFetching(false)
-    , m_nam(new QNetworkAccessManager(this))
 {
     bool fetchRatings = false;
     const QUrl ratingsUrl(QStringLiteral(APIURL "/ratings"));
@@ -82,7 +83,7 @@ void OdrsReviewsBackend::ratingsFetched(KJob *job)
 {
     m_isFetching = false;
     if (job->error()) {
-        qWarning() << "Failed to fetch ratings " << job->errorString();
+        qCWarning(LIBDISCOVER_LOG) << "Failed to fetch ratings " << job->errorString();
     } else {
         parseRatings();
     }
@@ -90,32 +91,7 @@ void OdrsReviewsBackend::ratingsFetched(KJob *job)
 
 static QString osName()
 {
-    QString osReleaseFilename;
-    if (QFileInfo::exists(QStringLiteral("/etc/os-release"))) {
-        osReleaseFilename = QStringLiteral("/etc/os-release");
-    } else if (QFileInfo::exists(QStringLiteral("/usr/lib/os-release"))) {
-        osReleaseFilename = QStringLiteral("/usr/lib/os-release");
-    }
-
-    if (osReleaseFilename.isEmpty()) {
-        return QStringLiteral("Unknown");
-    }
-
-    QFile osReleaseFile(osReleaseFilename);
-    if (osReleaseFile.open(QIODevice::ReadOnly)) {
-        QString line;
-        QTextStream stream(&osReleaseFile);
-        while (stream.readLineInto(&line)) {
-            if (line.startsWith(QStringLiteral("NAME"))) {
-                QStringRef name = line.midRef(5).trimmed();
-                if (name.startsWith(QLatin1Char('\"')) && name.endsWith(QLatin1Char('\"')))
-                    name = name.mid(1, name.size()-2);
-                return name.toString();
-            }
-        }
-    }
-
-    return QStringLiteral("Unknown");
+    return AppStreamIntegration::global()->osRelease()->name();
 }
 
 static QString userHash()
@@ -131,7 +107,7 @@ static QString userHash()
         return QString();
     }
 
-    QString salted = QStringLiteral("gnome-software[%1:%2]").arg(KUser().loginName()).arg(machineId);
+    QString salted = QStringLiteral("gnome-software[%1:%2]").arg(KUser().loginName(), machineId);
     return QString::fromUtf8(QCryptographicHash::hash(salted.toUtf8(), QCryptographicHash::Sha1).toHex());
 }
 
@@ -162,7 +138,7 @@ void OdrsReviewsBackend::fetchReviews(AbstractResource *app, int page)
             {QStringLiteral("user_hash"), userHash()},
             {QStringLiteral("version"), app->isInstalled() ? app->installedVersion() : app->availableVersion()},
             {QStringLiteral("locale"), QLocale::system().name()},
-            {QStringLiteral("limit"), 0}
+            {QStringLiteral("limit"), -1}
     });
 
     QNetworkRequest request(QUrl(QStringLiteral(APIURL "/fetch")));
@@ -171,44 +147,44 @@ void OdrsReviewsBackend::fetchReviews(AbstractResource *app, int page)
     // Store reference to the app for which we request reviews
     request.setOriginatingObject(app);
 
-    auto reply = m_nam->post(request, document.toJson());
+    auto reply = nam()->post(request, document.toJson());
     connect(reply, &QNetworkReply::finished, this, &OdrsReviewsBackend::reviewsFetched);
 }
 
 void OdrsReviewsBackend::reviewsFetched()
 {
     QNetworkReply* reply = qobject_cast<QNetworkReply*>(sender());
-
-    if (reply->error() == QNetworkReply::NoError) {
-        QByteArray data = reply->readAll();
-        const QJsonDocument document = QJsonDocument::fromJson(data);
-        AbstractResource *resource = qobject_cast<AbstractResource*>(reply->request().originatingObject());
-        Q_ASSERT(resource);
-        parseReviews(document, resource);
-
-        // Store reviews to cache so we don't need to download them all the time
-        if (document.array().isEmpty()) {
-            return;
-        }
-
-        QJsonObject jsonObject = document.array().first().toObject();
-        if (jsonObject.isEmpty()) {
-            return;
-        }
-
-        QFile file(QStandardPaths::writableLocation(QStandardPaths::CacheLocation) + QStringLiteral("/reviews/%1.json").arg(jsonObject.value(QStringLiteral("app_id")).toString()));
-        if (file.open(QIODevice::WriteOnly)) {
-            file.write(document.toJson());
-            file.close();
-        }
-    } else {
+    QScopedPointer<QNetworkReply, QScopedPointerDeleteLater> replyPtr(reply);
+    if (reply->error() != QNetworkReply::NoError) {
+        qCWarning(LIBDISCOVER_LOG) << "error fetching reviews:" << reply->errorString();
         m_isFetching = false;
+        return;
+    }
+
+    QByteArray data = reply->readAll();
+    const QJsonDocument document = QJsonDocument::fromJson(data);
+    AbstractResource *resource = qobject_cast<AbstractResource*>(reply->request().originatingObject());
+    Q_ASSERT(resource);
+    parseReviews(document, resource);
+    // Store reviews to cache so we don't need to download them all the time
+    if (document.array().isEmpty()) {
+        return;
+    }
+
+    QJsonObject jsonObject = document.array().first().toObject();
+    if (jsonObject.isEmpty()) {
+        return;
+    }
+
+    QFile file(QStandardPaths::writableLocation(QStandardPaths::CacheLocation) + QStringLiteral("/reviews/%1.json").arg(jsonObject.value(QStringLiteral("app_id")).toString()));
+    if (file.open(QIODevice::WriteOnly)) {
+        file.write(document.toJson());
     }
 }
 
 Rating * OdrsReviewsBackend::ratingForApplication(AbstractResource *app) const
 {
-    if (app->isTechnical()) {
+    if (app->appstreamId().isEmpty()) {
         return nullptr;
     }
 
@@ -225,11 +201,11 @@ void OdrsReviewsBackend::submitUsefulness(Review *review, bool useful)
                      {QStringLiteral("review_id"), QJsonValue(double(review->id()))} //if we really need uint64 we should get it in QJsonValue
     });
 
-    QNetworkRequest request(QUrl(QStringLiteral(APIURL "/%1").arg(useful ? QStringLiteral("upvote") : QStringLiteral("downvote"))));
+    QNetworkRequest request(QUrl(QStringLiteral(APIURL) + (useful ? QLatin1String("/upvote") : QLatin1String("/downvote"))));
     request.setHeader(QNetworkRequest::ContentTypeHeader, QStringLiteral("application/json; charset=utf-8"));
     request.setHeader(QNetworkRequest::ContentLengthHeader, document.toJson().size());
 
-    auto reply = m_nam->post(request, document.toJson());
+    auto reply = nam()->post(request, document.toJson());
     connect(reply, &QNetworkReply::finished, this, &OdrsReviewsBackend::usefulnessSubmitted);
 }
 
@@ -238,10 +214,16 @@ void OdrsReviewsBackend::usefulnessSubmitted()
     QNetworkReply* reply = qobject_cast<QNetworkReply*>(sender());
 
     if (reply->error() == QNetworkReply::NoError) {
-        qWarning() << "Usefullness submitted";
+        qCWarning(LIBDISCOVER_LOG) << "Usefullness submitted";
     } else {
-        qWarning() << "Failed to submit usefulness: " << reply->errorString();
+        qCWarning(LIBDISCOVER_LOG) << "Failed to submit usefulness: " << reply->errorString();
     }
+    reply->deleteLater();
+}
+
+QString OdrsReviewsBackend::userName() const
+{
+    return i18n("%1 (%2)", KUser().property(KUser::FullName).toString(), KUser().loginName());
 }
 
 void OdrsReviewsBackend::submitReview(AbstractResource *res, const QString &summary, const QString &description, const QString &rating)
@@ -259,7 +241,7 @@ void OdrsReviewsBackend::submitReview(AbstractResource *res, const QString &summ
 
     const QJsonDocument document(map);
 
-    QNetworkAccessManager *accessManager = new QNetworkAccessManager(this);
+    QNetworkAccessManager *accessManager = nam();
     QNetworkRequest request(QUrl(QStringLiteral(APIURL "/submit")));
     request.setHeader(QNetworkRequest::ContentTypeHeader, QStringLiteral("application/json; charset=utf-8"));
     request.setHeader(QNetworkRequest::ContentLengthHeader, document.toJson().size());
@@ -277,17 +259,18 @@ void OdrsReviewsBackend::submitReview(AbstractResource *res, const QString &summ
 void OdrsReviewsBackend::reviewSubmitted(QNetworkReply *reply)
 {
     if (reply->error() == QNetworkReply::NoError) {
-        qWarning() << "Review submitted";
+        qCWarning(LIBDISCOVER_LOG) << "Review submitted";
         AbstractResource *resource = qobject_cast<AbstractResource*>(reply->request().originatingObject());
         const QJsonArray array = {resource->getMetadata(QStringLiteral("ODRS::review_map")).toObject()};
-        QJsonDocument document = QJsonDocument(array);
+        const QJsonDocument document(array);
         // Remove local file with reviews so we can re-download it next time to get our review
         QFile file(QStandardPaths::writableLocation(QStandardPaths::CacheLocation) + QStringLiteral("/reviews/%1.json").arg(array.first().toObject().value(QStringLiteral("app_id")).toString()));
         file.remove();
         parseReviews(document, resource);
     } else {
-        qWarning() << "Failed to submit review: " << reply->errorString();
+        qCWarning(LIBDISCOVER_LOG) << "Failed to submit review: " << reply->errorString();
     }
+    reply->deleteLater();
 }
 
 void OdrsReviewsBackend::parseRatings()
@@ -295,7 +278,8 @@ void OdrsReviewsBackend::parseRatings()
     QFile ratingsDocument(QStandardPaths::writableLocation(QStandardPaths::CacheLocation) + QStringLiteral("/ratings/ratings"));
     if (ratingsDocument.open(QIODevice::ReadOnly)) {
         QJsonDocument jsonDocument = QJsonDocument::fromJson(ratingsDocument.readAll());
-        QJsonObject jsonObject = jsonDocument.object();
+        const QJsonObject jsonObject = jsonDocument.object();
+        m_ratings.reserve(jsonObject.size());
         for (auto it = jsonObject.begin(); it != jsonObject.end(); it++) {
             QJsonObject appJsonObject = it.value().toObject();
 
@@ -308,6 +292,7 @@ void OdrsReviewsBackend::parseRatings()
                                       { QStringLiteral("star5"), appJsonObject.value(QLatin1String("star5")).toInt() } };
 
             Rating *rating = new Rating(it.key(), ratingCount, ratingMap);
+            rating->setParent(this);
             m_ratings.insert(it.key(), rating);
         }
         ratingsDocument.close();
@@ -319,12 +304,16 @@ void OdrsReviewsBackend::parseRatings()
 void OdrsReviewsBackend::parseReviews(const QJsonDocument &document, AbstractResource *resource)
 {
     m_isFetching = false;
+    Q_ASSERT(resource);
+    if (!resource) {
+        return;
+    }
 
     QJsonArray reviews = document.array();
     if (!reviews.isEmpty()) {
         QVector<ReviewPtr> reviewList;
         for (auto it = reviews.begin(); it != reviews.end(); it++) {
-            QJsonObject review = it->toObject();
+            const QJsonObject review = it->toObject();
             if (!review.isEmpty()) {
                 const int usefulFavorable = review.value(QStringLiteral("karma_up")).toInt();
                 const int usefulTotal = review.value(QStringLiteral("karma_down")).toInt() + usefulFavorable;
@@ -356,3 +345,21 @@ bool OdrsReviewsBackend::isResourceSupported(AbstractResource* res) const
 {
     return !res->appstreamId().isEmpty();
 }
+
+void OdrsReviewsBackend::emitRatingFetched(AbstractResourcesBackend* b, const QList<AbstractResource *>& resources) const
+{
+    b->emitRatingsReady();
+    foreach(AbstractResource* res, resources) {
+        if (m_ratings.contains(res->appstreamId())) {
+            Q_EMIT res->ratingFetched();
+        }
+    }
+}
+
+QNetworkAccessManager * OdrsReviewsBackend::nam()
+{
+    if (!m_delayedNam) {
+        m_delayedNam = new QNetworkAccessManager(this);
+    }
+    return m_delayedNam;
+}
